/*
 * Copyright (c) 1990-1992, by Sun Microsystems, Inc.
 */

#include <sys/t_lock.h>
#include <sys/ddipropdefs.h>
#include <sys/mutex.h>
#include <sys/kstat.h>

/*
 * dev_info: 	The main device information structure this is intended to be
 *		opaque to drivers and drivers should use ddi functions to
 *		access *all* driver accessible fields.
 *
 * devi_parent_data includes property lists (interrupts, registers, etc.)
 * devi_driver_data includes whatever the driver wants to place there.
 */

struct dev_info  {

	struct dev_info *devi_parent;	/* my parent node in tree	*/
	struct dev_info *devi_child;	/* my child list head		*/
	struct dev_info *devi_sibling;	/* next element on my level	*/

	char	*devi_name;		/* device name			*/
	char	*devi_addr;		/* address part of name		*/
	int	devi_nodeid;		/* device nodeid		*/
	int	devi_instance;		/* device instance number	*/

	struct dev_ops *devi_ops;	/* driver operations		*/

	caddr_t	devi_parent_data;	/* parent private data		*/
	caddr_t	devi_driver_data;	/* driver private data		*/

	ddi_prop_t *devi_drv_prop_ptr;	/* head of driver prop list */
	ddi_prop_t *devi_sys_prop_ptr;	/* head of system prop list */

	struct ddi_minor_data *devi_minor;	/* head of minor list */
	struct dev_info *devi_next;	/* Next instance of this device */
	kmutex_t devi_lock;		/* Protects per-devinfo data */

	/* logical parents for busop primitives	 */

	struct dev_info *devi_bus_map_fault;	/* bus_map_fault parent */
	struct dev_info *devi_bus_dma_map;	/* bus_dma_map parent */
	struct dev_info *devi_bus_dma_ctl;	/* bus_dma_ctl parent */
	struct dev_info	*devi_bus_ctl;		/* bus_ctl parent */

	void	*devi_pm_info;		/* private power management stuff */

	/* some device driver statistical info may go here */
};

/*
 * ie.h header for STREAMS Intel 82586 Ethernet Driver.
 *
 * Per-Device instance state information.
 *
 * Each instance is dynamically allocated on first attach.
 */
struct	ie {
	struct	ie		*ie_nextp;	/* next in a linked list */
	dev_info_t		*ie_dip;	/* associated dev_info */
	struct	ether_addr	ie_ouraddr;	/* individual address */
	u_int			ie_flags;	/* misc. flags */
	u_int			ie_wantw;	/* xmit: out of resources */
	u_int			ie_jamtime;	/* last jammed time in secs */
	int			ie_dogid;	/* id returned from timeout() */
	struct iecb		*ie_cbsyncp;	/* synchronous command block */

	caddr_t		ie_cb_base;	/* control block base address */
	u_int		ie_cb_size;	/* control block size in bytes */

	kmutex_t	ie_buflock;	/* protect per-device buffer pool */
	kmutex_t	ie_devlock;	/* protect other per-device data */

	volatile	struct iescp	*ie_scp;	/* SCP pointer */
	volatile	struct ieiscp	*ie_iscp;	/* ISCP pointer */
	volatile	struct iescb	*ie_scbp;	/* SCB pointer */
	volatile	caddr_t	ie_csr;		/* control & status register */
	volatile	caddr_t	ie_iocache;	/* IC cache xmit/recv lines */
	volatile	caddr_t	ie_ram;		/* RAM on 3E or IORAM on */
						/* Sun4/4XX */

	/* Receive Frame Descriptors */
	struct ierfd	*ie_rfdring;	/* rfd ring start */
	struct ierfd	*ie_rfdlim;	/* one past end of ring */
	volatile struct ierfd	*ie_rfdhd;	/* rfd pointed to by scb */
	volatile struct ierfd	*ie_rfdtl;	/* last rfd (for reclaiming) */

	/* Receive Buffer Descriptors */
	struct ierbd	*ie_rbdring;	/* rbd ring start */
	struct ierbd	*ie_rbdlim;	/* one past end of ring */
	struct ierbd	*ie_rbdhd;	/* first rbd pointed to by first rfd */
	struct ierbd	*ie_rbdtl;	/* last rbd (for reclaim purpose) */

	/* Transmit and Receive Buffers */
	struct  iebuf	*ie_bufbase;	/* buffers base address */
	struct	iebuf	**ie_buftab;	/* buffer pointer stack (filo) */
	u_int	ie_bufi;		/* index of top of the above stack */
	struct  iebuf	**ie_rbp;	/* iebuf associated with rbd */
	struct  iebuf	**ie_tbp;	/* iebuf associated with tbd */
	mblk_t  **ie_tmblk;		/* streams msg associated with tbd */

	/* Transmit Frame Descriptors */
	struct ietcb	*ie_tcbring;	/* tcb ring start */
	struct ietcb	*ie_tcblim;	/* one past end of ring */
	struct ietcb	*ie_tcbtl;	/* last tcb to transmit */
	struct ietcb	*ie_tcbclaimed;	/* last tcb claimed */

	/* Transmit Buffer Descriptors */
	struct ietbd	*ie_tbdring;	/* tbd ring start */
	struct ietbd	*ie_tbdlim;	/* one past end of ring */

	ddi_dma_handle_t ie_dscpthandle;	/* iopb dma handle */
	ddi_dma_handle_t ie_bufhandle;		/* bufs dma handle */

	ulong_t	ie_ipackets;		/* # packets received */
	ulong_t	ie_ierrors;		/* # total input errors */
	ulong_t	ie_opackets;		/* # packets sent */
	ulong_t	ie_oerrors;		/* # total output errors */
	ulong_t	ie_collisions;		/* # collisions */
	ulong_t	ie_xmiturun;		/* # transmit underrun */
	ulong_t	ie_defer;		/* # defers */
	ulong_t	ie_heart;		/* # hearts */
	ulong_t	ie_crc;			/* # receive crc errors */
	ulong_t	ie_align;		/* # alignment errors */
	ulong_t	ie_recvorun;		/* # receive overrun */
	ulong_t	ie_discard;		/* # discarded packets */
	ulong_t	ie_oflo;		/* # receiver overflows */
	ulong_t	ie_uflo;		/* # transmit underflows */
	ulong_t	ie_missed;		/* # receive missed */
	ulong_t	ie_tlcol;		/* # transmit late collisions */
	ulong_t	ie_trtry;		/* # transmit retry failures */
	ulong_t	ie_tnocar;		/* # loss of carrier errors */
	ulong_t	ie_tnocts;		/* # loss of clear to send */
	ulong_t	ie_tbuff;		/* # BUFF set in tmd */
	ulong_t	ie_tsync;		/* # out of synch OWN bit occurences */
	ulong_t	ie_inits;		/* # driver inits */
	ulong_t	ie_notcbs;		/* # out of tcbs occurences */
	ulong_t	ie_notbufs;		/* # out of buffers for xmit */
	ulong_t	ie_norbufs;		/* # out of buffers for receive */
	ulong_t	ie_runotready;		/* # receive unit not ready */
	ulong_t	ie_nocanput;		/* # input canput() returned false */
	ulong_t	ie_allocbfail;		/* # esballoc/allocb failed */
	ulong_t	ie_loaned;		/* # receive biffer loaned */
	ulong_t	ie_dogreset;		/* # reset by iedog() */
};

/* flags */
#define	IERUNNING	0x01	/* chip is initialized */
#define	IESLAVE		0x02	/* slave device (no DMA) */
#define	IEPROMISC	0x04	/* promiscuous mode enabled */

/*
 * le.h header for LANCE Ethernet Driver.
 *
 * Per-Device instance state information.
 *
 * Each instance is dynamically allocated on first attach.
 */
struct	le {
	struct	le		*le_nextp;	/* next in a linked list */
	volatile	struct	lanceregs	*le_regsp;	/* chip regs */
	u_int	le_flags;			/* misc. flags */
	dev_info_t	*le_dip;	/* associated dev_info */
	struct	ether_addr	le_ouraddr;	/* individual address */

	kmutex_t	le_xmitlock;		/* protect xmit-side fields */
	kmutex_t	le_intrlock;		/* protect intr-side fields */
	kmutex_t	le_buflock;		/* protect private buffers */

	struct	lance_init_block *le_ibp;	/* chip init block */

	struct	lmd	*le_rmdp;	/* receive descriptor ring start */
	struct	lmd	*le_rmdlimp;	/* receive descriptor ring end */
	int	le_nrmdp2;		/* log(2) # receive descriptors */
	int	le_nrmds;		/* # receive descriptors */
	struct	lmd	*le_tmdp;	/* transmit descriptor ring start */
	struct	lmd	*le_tmdlimp;	/* transmit descriptor ring end */
	int	le_ntmdp2;		/* log(2) # transmit descriptors */
	int	le_ntmds;		/* # transmit descriptors */
	volatile	struct	lmd	*le_rnextp;	/* next chip rmd */
	volatile	struct	lmd	*le_tnextp;	/* next free tmd */
	volatile	struct	lmd	*le_tcurp;	/* next reclaim tmd */
	u_int	le_wantw;			/* xmit: out of resources */

	struct	lebuf	*le_tbufp[128];	/* xmit completion tbufs */
	mblk_t	*le_tmblkp[128];	/* xmit completion msgs */
	struct	lebuf	*le_rbufp[128];	/* lebuf associated with RMD */

	u_long	le_membase;		/* base address of slave card memory */
	int	le_memsize;		/* size of slave card memory in bytes */
	int	(*le_init)();		/* device-specific initialize */
	int	(*le_intr)();		/* device-specific interrupt */
	caddr_t	le_arg;			/* device-specific arg */

	int	le_nbufs;		/* # buffers */
	caddr_t	le_bufbase;		/* buffers base address */
	caddr_t	le_buflim;		/* buffers limit address */
	struct	lebuf	**le_buftab;	/* buffer pointer stack (fifo) */
	int	le_bufi;		/* index of buffer ptr stack top */

	int	le_tpe;			/* =1 => tpe selected */
	int	le_autosel;		/* =1 => auto-selection enabled */
	int	le_oopkts;		/* old le_opackets */

	/*
	 * DDI dma handle, kernel virtual base,
	 * and io virtual base of IOPB area.
	 */
	ddi_dma_handle_t	le_iopbhandle;
	u_long	le_iopbkbase;
	u_long	le_iopbiobase;

	queue_t	*le_ipq;		/* ip read queue */

	/*
	 * DDI dma handle, kernel virtual base,
	 * and io virtual base addresses of buffer area.
	 */
	ddi_dma_handle_t	le_bufhandle;
	u_long	le_bufkbase;
	u_long	le_bufiobase;

	ulong_t	le_ipackets;		/* # packets received */
	ulong_t	le_ierrors;		/* # total input errors */
	ulong_t	le_opackets;		/* # packets sent */
	ulong_t	le_oerrors;		/* # total output errors */
	ulong_t	le_collisions;		/* # collisions */
	ulong_t	le_defer;		/* # defers */
	ulong_t	le_fram;		/* # receive framming errors */
	ulong_t	le_crc;			/* # receive crc errors */
	ulong_t	le_oflo;		/* # receiver overflows */
	ulong_t	le_uflo;		/* # transmit underflows */
	ulong_t	le_missed;		/* # receive missed */
	ulong_t	le_tlcol;		/* # transmit late collisions */
	ulong_t	le_trtry;		/* # transmit retry failures */
	ulong_t	le_tnocar;		/* # loss of carrier errors */
	ulong_t	le_inits;		/* # driver inits */
	ulong_t	le_notmds;		/* # out of tmds occurences */
	ulong_t	le_notbufs;		/* # out of buffers for xmit */
	ulong_t	le_norbufs;		/* # out of buffers for receive */
	ulong_t	le_nocanput;		/* # input canput() returned false */
	ulong_t	le_allocbfail;		/* # esballoc/allocb failed */
};

/* flags */
#define	LERUNNING	0x01	/* chip is initialized */
#define	LESLAVE		0x02	/* slave device (no DMA) */
#define	LEPROMISC	0x04	/* promiscuous mode enabled */
#define	LESUN4C		0x08	/* this system is a sun4c */
#define	LESUSPENDED	0x10	/* suspended interface */

/*
 * Declarations and definitions specific to the
 * Quad Ethernet Device (QED) Driver.
 *
 * QED Per-Channel instance state information.
 *
 * Each instance is dynamically allocated on first attach.
 */
#define QEC_QMDMAX      (256)                   /* # Tx/Rx ring entries */

struct	qe {
	struct	qe		*qe_nextp;	/* next in a linked list */
	dev_info_t		*qe_dip;	/* associated dev_info */
	struct	ether_addr	qe_ouraddr;	/* individual address */
	int			qe_chan;	/* channel no */
	u_int			qe_flags;	/* misc. flags */
	u_int			qe_wantw;	/* xmit: out of resources */
	ddi_iblock_cookie_t	qe_cookie;	/* cookie from ddi_add_intr */

	volatile struct	qecm_chan	*qe_chanregp;	/* QEC chan regs */
	volatile struct	mace		*qe_maceregp;	/* MACE regs */

	kmutex_t	qe_xmitlock;		/* protect xmit-side fields */
	kmutex_t	qe_intrlock;		/* protect intr-side fields */
	kmutex_t	qe_buflock;		/* protect private buffers */

	struct	qmd	*qe_rmdp;	/* receive descriptor ring start */
	struct	qmd	*qe_rmdlimp;	/* receive descriptor ring end */
	struct	qmd	*qe_tmdp;	/* transmit descriptor ring start */
	struct	qmd	*qe_tmdlimp;	/* transmit descriptor ring end */
	volatile	struct	qmd	*qe_rnextp;	/* next chip rmd */
	volatile	struct	qmd	*qe_rlastp;	/* last free rmd */
	volatile	struct	qmd	*qe_tnextp;	/* next free tmd */
	volatile	struct	qmd	*qe_tcurp;	/* next tmd to reclm */

	struct	qebuf	*qe_tbufp[QEC_QMDMAX];	/* qebuf associated with TMD */
	struct	qebuf	*qe_rbufp[QEC_QMDMAX];	/* qebuf associated with RMD */
	mblk_t 	*qe_tmblkp[QEC_QMDMAX];		/* qebuf associated with TMD */

	caddr_t	qe_buflim;		/* buffers limit address */
	struct	qebuf	**qe_buftab;	/* buffer pointer stack (fifo) */
	int	qe_bufi;		/* index of buffer ptr stack top */
	queue_t	*qe_ipq;		/* ip read queue */

	/*
	 * DDI dma handle, kernel virtual base,
	 * and io virtual base of IOPB area.
	 */
	ddi_dma_handle_t	qe_iopbhandle;
	u_long			qe_iopbkbase;
	u_long			qe_iopbiobase;

	/*
	 * DDI dma handle, kernel virtual base,
	 * and io virtual base addresses of buffer area.
	 */
	ddi_dma_handle_t	qe_bufhandle;
	u_long			qe_bufkbase;
	u_long			qe_bufiobase;

#ifndef	MPSAS
	kstat_t	*qe_ksp;	/* kstat pointer */
#endif	MPSAS
	u_long	qe_ipackets;	/* # packets received */
	u_long	qe_ierrors;	/* # total input errors */
	u_long	qe_opackets;	/* # packets sent */
	u_long	qe_oerrors;	/* # total output errors */
	u_long	qe_txcoll;	/* # xmit collisions */
	u_long	qe_rxcoll;	/* # recv collisions */
	u_long	qe_defer;	/* # excessive defers */
	u_long	qe_fram;	/* # recv framing errors */
	u_long	qe_crc;		/* # recv crc errors */
	u_long	qe_buff;	/* # recv packet sizes > buffer size */
	u_long	qe_drop;	/* # recv packets dropped */
	u_long	qe_oflo;	/* # recv overflow */
	u_long	qe_uflo;	/* # xmit underflow */
	u_long	qe_missed;	/* # recv missed */
	u_long	qe_tlcol;	/* # xmit late collision */
	u_long	qe_trtry;	/* # xmit retry failures */
	u_long	qe_tnocar;	/* # loss of carrier errors */
	u_long	qe_inits;	/* # driver inits */
	u_long	qe_nocanput;	/* # canput() failures */
	u_long	qe_allocbfail;	/* # allocb() failures */
	u_long	qe_runt;	/* # recv runt packets */
	u_long	qe_jab;		/* # mace jabber errors */
	u_long	qe_babl;	/* # mace babble errors */
	u_long	qe_tmder;	/* # chained tx desc. errors */
	u_long	qe_laterr;	/* # sbus tx late error */
	u_long	qe_parerr;	/* # sbus tx parity errors */
	u_long	qe_errack;	/* # sbus tx error acks */
	u_long	qe_notmds;	/* # out of tmds */
	u_long	qe_notbufs;	/* # out of xmit buffers */
	u_long	qe_norbufs;	/* # out of recv buffers */
	u_long	qe_clsn;	/* # recv late collisions */

};

/* flags */
#define	QERUNNING	0x01	/* chip is initialized */
#define	QEPROMISC	0x02	/* promiscuous mode enabled */
#define	QESUN4C		0x04	/* this system is a sun4c */

